@@ -3,7 +3,10 @@ use {
     console::Emoji,
     indicatif::{ProgressBar, ProgressStyle},
     log::*,
-    solana_runtime::{bank_forks::ArchiveFormat, snapshot_utils},
+    solana_runtime::{
+        snapshot_package::SnapshotType,
+        snapshot_utils::{self, ArchiveFormat},
+    },
     solana_sdk::{clock::Slot, genesis_config::DEFAULT_GENESIS_ARCHIVE, hash::Hash},
     std::{
         fs::{self, File},
@@ -81,7 +84,7 @@ pub fn download_file<'a, 'b>(
 
     let progress_bar = new_spinner_progress_bar();
     if use_progress_bar {
-        progress_bar.set_message(&format!("{}Downloading {}...", TRUCK, url));
+        progress_bar.set_message(format!("{}Downloading {}...", TRUCK, url));
     }
 
     let response = reqwest::blocking::Client::new()
@@ -111,7 +114,7 @@ pub fn download_file<'a, 'b>(
                 )
                 .progress_chars("=> "),
         );
-        progress_bar.set_message(&format!("{}Downloading~ {}", TRUCK, url));
+        progress_bar.set_message(format!("{}Downloading~ {}", TRUCK, url));
     } else {
         info!("Downloading {} bytes from {}", download_size, url);
     }
@@ -247,29 +250,49 @@ pub fn download_genesis_if_missing(
     }
 }
 
-pub fn download_snapshot<'a, 'b>(
+/// Download a snapshot archive from `rpc_addr`.  Use `snapshot_type` to specify downloading either
+/// a full snapshot or an incremental snapshot.
+pub fn download_snapshot_archive<'a, 'b>(
     rpc_addr: &SocketAddr,
-    snapshot_output_dir: &Path,
+    snapshot_archives_dir: &Path,
     desired_snapshot_hash: (Slot, Hash),
+    snapshot_type: SnapshotType,
+    maximum_full_snapshot_archives_to_retain: usize,
+    maximum_incremental_snapshot_archives_to_retain: usize,
     use_progress_bar: bool,
-    maximum_snapshots_to_retain: usize,
     progress_notify_callback: &'a mut DownloadProgressCallbackOption<'b>,
 ) -> Result<(), String> {
-    snapshot_utils::purge_old_snapshot_archives(snapshot_output_dir, maximum_snapshots_to_retain);
+    snapshot_utils::purge_old_snapshot_archives(
+        snapshot_archives_dir,
+        maximum_full_snapshot_archives_to_retain,
+        maximum_incremental_snapshot_archives_to_retain,
+    );
 
-    for compression in &[
+    for archive_format in [
         ArchiveFormat::TarZstd,
         ArchiveFormat::TarGzip,
         ArchiveFormat::TarBzip2,
         ArchiveFormat::Tar, // `solana-test-validator` creates uncompressed snapshots
     ] {
-        let desired_snapshot_package = snapshot_utils::get_snapshot_archive_path(
-            snapshot_output_dir.to_path_buf(),
-            &desired_snapshot_hash,
-            *compression,
-        );
+        let destination_path = match snapshot_type {
+            SnapshotType::FullSnapshot => snapshot_utils::build_full_snapshot_archive_path(
+                snapshot_archives_dir.to_path_buf(),
+                desired_snapshot_hash.0,
+                &desired_snapshot_hash.1,
+                archive_format,
+            ),
+            SnapshotType::IncrementalSnapshot(base_slot) => {
+                snapshot_utils::build_incremental_snapshot_archive_path(
+                    snapshot_archives_dir.to_path_buf(),
+                    base_slot,
+                    desired_snapshot_hash.0,
+                    &desired_snapshot_hash.1,
+                    archive_format,
+                )
+            }
+        };
 
-        if desired_snapshot_package.is_file() {
+        if destination_path.is_file() {
             return Ok(());
         }
 
@@ -277,13 +300,9 @@ pub fn download_snapshot<'a, 'b>(
             &format!(
                 "http://{}/{}",
                 rpc_addr,
-                desired_snapshot_package
-                    .file_name()
-                    .unwrap()
-                    .to_str()
-                    .unwrap()
+                destination_path.file_name().unwrap().to_str().unwrap()
             ),
-            &desired_snapshot_package,
+            &destination_path,
             use_progress_bar,
             progress_notify_callback,
         ) {
@@ -292,7 +311,7 @@ pub fn download_snapshot<'a, 'b>(
         }
     }
     Err(format!(
-        "Failed to download a snapshot for slot {} from {}",
+        "Failed to download a snapshot archive for slot {} from {}",
         desired_snapshot_hash.0, rpc_addr
     ))
 }
